<?php
namespace Teknisa\SUP\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\Report\ReportService;

class CadastroCategoriaSupplyTekController extends \Zeedhi\Framework\Controller\Report {

    protected $environment;
    protected $dataSourceManager;
    protected $crudService;
    protected $newCode;
    protected $entityManager;
    protected $service;
    protected $imgPathURL;
    protected $birtStrategy;
    protected $connection;

    public function __construct ($reportService,$environment, $dataSourceManager, $crudService, $newCode, $entityManager, $service, $imgPathURL,$birtStrategy,$connection) {
    	parent::__construct($reportService);
    	$this->environment = $environment;
    	$this->dataSourceManager = $dataSourceManager;
    	$this->crudService  = $crudService;
    	$this->newCode = $newCode;
        $this->entityManager = $entityManager;
        $this->service = $service;
        $this->imgPathURL = $imgPathURL;
        $this->birtStrategy = $birtStrategy;
        $this->connection  = $connection;
       
    }
    
    public function generateCode($newCode)
    {
        return $this->newCode->getCodeByTableName($newCode, $this->environment->getNrOrgTrab(), 1, 3);
    }
    
    public function saveCategory(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $row['CDCATEGORIA'] = self::generateCode('CATEGORIASUPPLY');
            $this->service->insertNewCategory($row);
            foreach ($row['NMSUBCATEGORIA'] as $subcateg) {
               $row['CDSUBCATEGORIA'] = self::generateCode('SUBCATEGSUPPLY');
               $row['NMSUBCATEGORIA'] = $subcateg;
               $this->service->insertNewSubCategory($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function saveChangeProducts($request, $response) 
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
            foreach ($dataSet as $row) {
                $this->service->updateProduct($row);
            }
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function saveNewProduct(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $produtos = $this->service->selectCdProduct($row);
            $messages = array();
            
            foreach ($produtos as $produto) {
                $check = false;
                $row["CDPRODUTO"]    = $produto["CDPRODUTO"];
                $row["NMPRODUTO"]    = $produto["NMPRODUTO"];
                $row["CDREGIAO"]     = $produto["CDREGIAO"];
                $row["CDMARCA"]      = $produto["CDMARCA"];
                $row["CDFILIAL"]     = $produto["CDFILIAL"];
                $row["IDATIVO"]      = $produto["IDATIVO"];
                $row["VRPRECUNIITM"] = $produto["VRPRECUNIITM"];

                $registeredProduct = $this->service->checkProductAlreadyRegistered($row);

                if (sizeof($registeredProduct) == 0) {
                    $check = true;
                    $this->service->insertNewProduct($row);
                }

                $this->createMessage($messages, $produto, $registeredProduct, $check);
            }
            
            sort($messages);
            $response->addDataSet(new DataSet("messages", $messages));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function createMessage(&$message, $product, $registeredProduct, $check)
    {
        switch ($check) {
            case true:
                $message[] = $product["NMPRODUTO"] . " incluído com sucesso. \n";
            break;
            
            case false:
                $message[] = $product["NMPRODUTO"] . " da região " . $registeredProduct["NMREGIAO"] . " não pode ser incluído porque ele já se encontra cadastrado em: " . $registeredProduct["NMCATEGORIA"] . " -> " . $registeredProduct["NMSUBCATEGORIA"] . "\n";
            break;
        }
    }
    
    public function deleteCategory($request, $response) 
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
            $categorias = array_filter($dataSet, function($row) {
                if($row['PAI'] !== NULL) {
                    $this->service->deleteSubCategory($row);
                    $this->service->deleteProdSubCategory($row);
                }
                return $row['PAI'] == NULL;
            });
            if(!empty($categorias)) {
                $this->service->deleteCategory($categorias);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteProduct($request, $response)
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
             foreach ($dataSet as $row) {
                $this->service->deleteProduct($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function insertSubCatInEdition($request, $response) 
    {
        try {
            $row = $request->getRow();
            foreach ($row['NEW_SUBCAT'] as $subcateg) {
               $row['CDSUBCATEGORIA'] = self::generateCode('SUBCATEGSUPPLY');
               $row['NMSUBCATEGORIA'] = $subcateg['NMSUBCATEGORIA'];
               $this->service->insertNewSubCategory($row);
            }
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function updateSubCategory($request, $response) 
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
            foreach ($dataSet as $row) {
                $this->service->updateSubCategory($row);
            }
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function selectCategory($request, $response) 
    {
        try {
            $rows = $request->getDataSet()->getRows();
            $retorno = $this->service->selectCategory($rows[0]);
            $response->addDataSet(new DataSet("resp", $retorno));
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function produtosSubCategoria(DTO\Request $request, DTO\Response $response) 
    {
		$filterCriteria = $request->getFilterCriteria();
		$filterCriteria->setDataSourceName("grid_produtos_supplyTek");
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		$newDataSet = $this->setImgFullPath($dataSet);
        $response->addDataSet($newDataSet);
	}
	
    private function setImgFullPath($dataSet) 
    {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["DSIDFOTOFILESERV"] !== null) {
				$row["DSIDFOTOFILESERV"] = $this->imgPathURL.$row["DSIDFOTOFILESERV"];
			}
		}
		return new DataSet("grid_produtos_supplyTek", $rows);
	}

     protected $reportMapping = array (
       'CATALOGO_PRODUTO_UNIDADE' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        
                        'P_CDFILIAL'       =>'P_CDFILIAL',
                        'P_CDFORNECED'    =>'P_CDFORNECED',
                        'P_CDCATEGORIA'   =>'P_CDCATEGORIA',
                        'P_CDSUBCATEGORIA'=>'P_CDSUBCATEGORIA',
                        'P_CDREGIAO'      =>'P_CDREGIAO',
                        'CDARVPRODINI'  =>'P_CDARVPRODINI',
                        'CDARVPRODFIN'  =>'P_CDARVPRODFIN',
                        'P_TCDARVPROD'  =>'P_TCDARVPROD',
                        'IDCONSPROD'    =>'P_IDCONSPROD',
                        'NRORG'         =>'P_NRORG',
                    ),
            ),
         'CATALOGO_PRODUTO_FOTO' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        
                        'P_CDFILIAL'      =>'P_CDFILIAL',
                        'P_CDFORNECED'    =>'P_CDFORNECED',
                        'P_CDCATEGORIA'   =>'P_CDCATEGORIA',
                        'P_CDSUBCATEGORIA'=>'P_CDSUBCATEGORIA',
                        'P_CDREGIAO'      =>'P_CDREGIAO',
                        'CDARVPRODINI'    =>'P_CDARVPRODINI',
                        'CDARVPRODFIN'    =>'P_CDARVPRODFIN',
                        'P_TCDARVPROD'    =>'P_TCDARVPROD',
                        'IDCONSPROD'      =>'P_IDCONSPROD',
                        'URL'           =>'P_URL',
                        'NRORG'           =>'P_NRORG',
                    ),
            ),
    );
     public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
         if($row['P_CDCATEGORIA'] == ""){
            $row['P_CDCATEGORIA'] ='T';
         }
         if($row['P_CDSUBCATEGORIA'] == "" ){
            $row['P_CDSUBCATEGORIA']  ='T';
        }
         if($row['P_CDFILIAL'] == ""){
            $row['P_CDFILIAL'] ='T';
        }
         if($row['P_CDFORNECED'] == ""){
              $row['P_CDFORNECED'] ='T';
        }
         if($row['P_CDREGIAO'] == ""){
            $row['P_CDREGIAO']  ='T';
        }
          if($row['NMPRODINI'] == ""){
              $row['CDARVPRODINI']=null;
              $row['CDARVPRODFIN']=null;
              $row['P_TCDARVPROD']='T';
          }
          else{
              $row['P_TCDARVPROD']=null;
          }
          $row['NRORG'] = $this->environment->getNrOrg();
           $row["URL"]=$this->imgPathURL;
    }
}